//----------------------------------------------------------------------------
//
// File:        cartridge.hpp
// Date:        23-Feb-1998
// Programmer:  Marc Rousseau
//
// Description:
//
// Copyright (c) 1998-2000 Marc Rousseau, All Rights Reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
//
// Revision History:
//
//----------------------------------------------------------------------------

#ifndef _CARTRIDGE_HPP_
#define _CARTRIDGE_HPP_

#if defined ( ADDRESS )
#undef ADDRESS
#endif

typedef unsigned short ADDRESS;

enum eMemoryType { _RAM_, _ROM_, _BACKED_RAM_ };

enum eMemoryRegion {
    ROM_0,		// 0x0000 - 0x0FFF
    ROM_1,		// 0x1000 - 0x1FFF
    ROM_2,		// 0x2000 - 0x2FFF
    ROM_3,		// 0x3000 - 0x3FFF
    ROM_4,		// 0x4000 - 0x4FFF
    ROM_5,		// 0x5000 - 0x5FFF
    ROM_6,		// 0x6000 - 0x6FFF
    ROM_7,		// 0x7000 - 0x7FFF
    ROM_8,		// 0x8000 - 0x8FFF
    ROM_9,		// 0x9000 - 0x9FFF
    ROM_A,		// 0xA000 - 0xAFFF
    ROM_B,		// 0xB000 - 0xBFFF
    ROM_C,		// 0xC000 - 0xCFFF
    ROM_D,		// 0xD000 - 0xDFFF
    ROM_E,		// 0xE000 - 0xEFFF
    ROM_F,		// 0xF000 - 0xFFFF
    GROM_0,		// 0x0000 - 0x1FFF
    GROM_1,		// 0x2000 - 0x3FFF
    GROM_2,		// 0x4000 - 0x5FFF
    GROM_3,		// 0x6000 - 0x7FFF
    GROM_4,		// 0x8000 - 0x9FFF
    GROM_5,		// 0xA000 - 0xBFFF
    GROM_6,		// 0xC000 - 0xDFFF
    GROM_7 		// 0xE000 - 0xFFFF
};

struct sMemoryRegion {
    eMemoryType Type;
    USHORT      cruBase;
    USHORT      noBanks;
    USHORT      noBytes[4];
    UCHAR      *curBank;
    UCHAR      *Bank[4];
};

class cCartridge {

    static const char  *sm_Banner;

public:

    char               *m_Title;

    sMemoryRegion CpuMemory [16];
    sMemoryRegion GromMemory [8];

    cCartridge ( char * = NULL );
    ~cCartridge ();

    bool IsValid () const;

    void LoadImage ( char * );
    void SaveImage ( char * );
};

#endif
