//----------------------------------------------------------------------------
//
// File:        tms9900.hpp
// Date:        23-Feb-1998
// Programmer:  Marc Rousseau
//
// Description:
//
// Copyright (c) 1998-2000 Marc Rousseau, All Rights Reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
//
// Revision History:
//
//----------------------------------------------------------------------------

#ifndef _TMS9900_HPP_
#define _TMS9900_HPP_

#if defined ( ADDRESS )
    #undef ADDRESS
#endif

typedef unsigned short ADDRESS;

typedef USHORT (*TRAP_FUNCTION) ( void *, int, bool, const ADDRESS, USHORT );

#define MEMFLG_READ	0x80
#define MEMFLG_WRITE	0x40
#define MEMFLG_ACCESS	0xC0
#define MEMFLG_ROM	0x20

enum eMemoryAccess { MEM_ROM, MEM_RAM };

struct sOpCode {
    char        mnemonic [8];
    USHORT      opCode;
    USHORT      mask;
    USHORT      format;
    USHORT      unused;
    void        (*function) ();
    ULONG       clocks;
    ULONG       count;
};

struct sLookUp {
    sOpCode    *opCode;
    int         size;
};

struct sTrapInfo {
    void          *ptr;
    int            data;
    TRAP_FUNCTION  function;
};

union STATUS {
    USHORT value;
    struct {
        USHORT IntMask   : 4;
        USHORT reserved  : 5;
        USHORT XOP       : 1;
        USHORT OddParity : 1;
        USHORT Overflow  : 1;
        USHORT Carry     : 1;
        USHORT Equal     : 1;
        USHORT ArithGt   : 1;
        USHORT LogicGt   : 1;
    } flags;
};

class cTMS9900 {

/**/
#if defined ( NO_ASM )

    USHORT    WP;
    USHORT    PC;
    STATUS    ST;

    long      srcComp;
    long      dstComp;
    static bool      flagLogical;
    static bool      flagArithmetic;
    static bool      flagEqual;
    static bool      flagCarry;
    static bool      flagOverflow;
    static bool      flagParity;
    static bool      flagXOP;
    static bool      intLevel;

    UCHAR     InterruptFlag;
    USHORT    prefetchOK;
    USHORT    curOpCode;
    USHORT    arg1;
    USHORT    arg2;
    USHORT    size;

    ULONG     InstructionCounter;

    USHORT GetSrcAddress ();
    USHORT GetDestAddress ();

    USHORT Prefetch ();

    void CompareB ( UCHAR arg1, UCHAR arg2 );
    void CompareW ( USHORT arg1, USHORT arg2 );

    UCHAR  _ReadMemoryB_ ( ADDRESS address );
    USHORT _ReadMemoryW_ ( ADDRESS address );

    void  _WriteMemoryB_ ( ADDRESS address, UCHAR data );
    void  _WriteMemoryW_ ( ADDRESS address, USHORT data );

    UCHAR  ReadMemoryB ( ADDRESS address )			{ return CpuMemory [ address ]; }
    USHORT ReadMemoryW ( ADDRESS address )			{ return ( CpuMemory [ address ] << 8 ) | CpuMemory [ address + 1 ]; }

    void  WriteMemoryB ( ADDRESS address, UCHAR data )		{ CpuMemory [ address ] = data; }
    void  WriteMemoryW ( ADDRESS address, USHORT data )		{ CpuMemory [ address ] = data >> 8; CpuMemory [ address + 1 ] = data; }

    UCHAR  ReadRegisterB ( int reg )				{ return ReadMemoryB ( WP + reg * 2 ); }
    USHORT ReadRegisterW ( int reg )				{ return ReadMemoryW ( WP + reg * 2 ); }

    void   WriteRegisterB ( int reg, UCHAR data )		{ WriteMemoryB ( WP + reg * 2, data ); }
    void   WriteRegisterW ( int reg, USHORT data )		{ WriteMemoryW ( WP + reg * 2, data ); }

    void CheckInterrupt ();

    void ContextSwitch ( ADDRESS, ADDRESS );
    void ExecuteInstruction ();

    USHORT Breakpoint ( USHORT, int, USHORT );

#include "opcodes.hpp"

#endif
/**/

    static int sortFunction ( const sOpCode *p1, const sOpCode *p2 );

public:

    cTMS9900 ();
    ~cTMS9900 ();

    void SetPC ( ADDRESS address );
    void SetWP ( ADDRESS address );
    void SetST ( USHORT  address );

    ADDRESS GetPC ();
    ADDRESS GetWP ();
    USHORT  GetST ();

/**/
#if defined ( NO_ASM )
    void PrefetchInstruction ();
#endif
/**/

    void Run ();
    void Stop ();
    bool Step ();

    bool IsRunning ();

    void SignalInterrupt ( UCHAR );

    ULONG GetClocks ();
    void  ResetClocks ();
    ULONG GetCounter ();
    void  ResetCounter ();

    void SaveImage ( FILE * );
    void LoadImage ( FILE * );

    UCHAR RegisterBreakpoint ( TRAP_FUNCTION function, void *ptr, int data );
    void  DeRegisterBreakpoint ( UCHAR );

    UCHAR GetBreakpoint ( TRAP_FUNCTION function, int data );
    int   SetBreakpoint ( ADDRESS address, UCHAR type, int byte, UCHAR index );
    void  SetMemory ( eMemoryAccess, ADDRESS, long );

    void  ClearBreakpoint ( UCHAR index );

};

#endif
